@@ -14,6 +14,9 @@
 #' @export
 #' @param observations An \code{stslist} object (see \code{\link[TraMineR]{seqdef}}) containing
 #'   the sequences, or a list of such objects (one for each channel).
+#' @param n_states A numerical vector giving the number of hidden states in each submodel 
+#' (not used if starting values for model parameters are given with 
+#' \code{initial_probs}, \code{transition_probs}, and \code{emission_probs}).
 #' @param transition_probs A list of matrices of transition
 #'   probabilities for the submodel of each cluster.
 #' @param emission_probs A list which contains matrices of emission probabilities or
@@ -38,6 +41,7 @@
 #' the state names are taken as row names of transition matrices. If this is also \code{NULL},
 #' numbered states are used.
 #' @param channel_names A vector of optional names for the channels.
+#' @param ... Additional arguments to \code{simulate_transition_probs}.
 #' @return Object of class \code{mhmm} with following elements:
 #' \describe{
 #'    \item{\code{observations}}{State sequence object or a list of such containing the data.}
@@ -80,6 +84,14 @@
 #' attr(child_seq, "cpal") <- c("#66C2A5", "#FC8D62")
 #' attr(left_seq, "cpal") <- c("#A6CEE3", "#E31A1C")
 #'
+#' ## MHMM with random starting values, no covariates
+#' set.seed(468)
+#' init_mhmm_bf1 <- build_mhmm(
+#'   observations = list(marr_seq, child_seq, left_seq),
+#'   n_states = c(4, 4, 6),
+#'   channel_names = c("Marriage", "Parenthood", "Residence"))
+#'   
+#'   
 #' ## Starting values for emission probabilities
 #'
 #' # Cluster 1
@@ -182,8 +194,8 @@
 #' biofam3c$covariates$cohort <- factor(
 #'   biofam3c$covariates$cohort, labels=c("1909-1935", "1936-1945", "1946-1957"))
 #'
-#' # Build mixture HMM
-#' init_mhmm_bf <- build_mhmm(
+#' ## MHMM with own starting values and covariates
+#' init_mhmm_bf2 <- build_mhmm(
 #'   observations = list(marr_seq, child_seq, left_seq),
 #'   initial_probs = list(initial_probs1, initial_probs1, initial_probs2),
 #'   transition_probs = list(A1, A1, A2),
@@ -197,173 +209,295 @@
 #'                      paste("State", 1:6)))
 #'
 build_mhmm <-
-  function(observations,transition_probs,emission_probs,initial_probs,
-           formula, data, coefficients, cluster_names=NULL, state_names=NULL, channel_names=NULL){
-
-    if (is.list(transition_probs)){
-      n_clusters<-length(transition_probs)
-    }else{
-      stop("Transition_probs is not a list.")
-    }
-    if(length(emission_probs)!=n_clusters || length(initial_probs)!=n_clusters)
-      stop("Unequal list lengths of transition_probs, emission_probs and initial_probs.")
-
-    if(is.null(cluster_names)){
-      cluster_names <- paste("Cluster", 1:n_clusters)
-    }else if(length(cluster_names)!=n_clusters){
-      warning("The length of argument cluster_names does not match the number of clusters. Names were not used.")
-      cluster_names <- paste("Cluster", 1:n_clusters)
-    }
-
-
-    for(i in 1:n_clusters){
-      if (!is.matrix(transition_probs[[i]])) {
-        stop(paste("Object provided in transition_probs for cluster", i, "is not a matrix."))
+  function(observations, n_states, transition_probs, emission_probs, initial_probs,
+           formula, data, coefficients, cluster_names = NULL, state_names = NULL, 
+           channel_names = NULL, ...){
+    
+    
+    
+    if (missing(n_states)) {
+      
+      if (missing(transition_probs) || missing(initial_probs) || missing(emission_probs)) {
+        stop(paste("Provide either n_states or all three of initial_probs, transition_probs, and emission_probs."))
       }
-      if (!is.vector(initial_probs[[i]])) {
-        stop(paste("Object provided in initial_probs for cluster", i, "is not a vector."))
+      
+      if (is.list(transition_probs)){
+        n_clusters<-length(transition_probs)
+      }else{
+        stop("Transition_probs is not a list.")
       }
-    }
-
-    # States
-    n_states <- unlist(lapply(transition_probs,nrow))
-
-    if (any(rep(n_states, each = 2) != unlist(lapply(transition_probs, dim)))) {
-      stop("Transition matrices must be square matrices.")
-    }
-
-    if (is.null(state_names)) {
-      state_names <- vector("list", n_clusters)
-      for(m in 1:n_clusters){
-        if (is.null(state_names[[m]] <- rownames(transition_probs[[m]]))) {
-          state_names[[m]] <-  paste("State", 1:n_states[m])
+      if(length(emission_probs)!=n_clusters || length(initial_probs)!=n_clusters)
+        stop("Unequal list lengths of transition_probs, emission_probs and initial_probs.")
+      
+      if(is.null(cluster_names)){
+        cluster_names <- paste("Cluster", 1:n_clusters)
+      }else if(length(cluster_names)!=n_clusters){
+        warning("The length of argument cluster_names does not match the number of clusters. Names were not used.")
+        cluster_names <- paste("Cluster", 1:n_clusters)
+      }
+      
+      
+      for(i in 1:n_clusters){
+        if (!is.matrix(transition_probs[[i]])) {
+          stop(paste("Object provided in transition_probs for cluster", i, "is not a matrix."))
+        }
+        if (!is.vector(initial_probs[[i]])) {
+          stop(paste("Object provided in initial_probs for cluster", i, "is not a vector."))
         }
       }
-    } else {
-      for (m in 1:n_clusters) {
-        if (length(state_names[[m]]) != n_states[m]) {
-          stop(paste0("Length of state_names for cluster ", m, " is not equal to the number of hidden states."))
-        }
+      
+      # States
+      n_states <- unlist(lapply(transition_probs,nrow))
+      
+      if (any(rep(n_states, each = 2) != unlist(lapply(transition_probs, dim)))) {
+        stop("Transition matrices must be square matrices.")
       }
-    }
-
-    for(i in 1:n_clusters){
-      if (!isTRUE(all.equal(rowSums(transition_probs[[i]]),
-                            rep(1, n_states[i]), check.attributes=FALSE))) {
-        stop(paste("Row sums of the transition probabilities in cluster", i, "do not sum to one."))
-      }
-      if (!isTRUE(all.equal(sum(initial_probs[[i]]), 1, check.attributes=FALSE))){
-        stop(paste("Initial state probabilities in cluster", i, "do not sum to one."))
-      }
-    }
-
-
-
-
-    for(i in 1:n_clusters){
-      dimnames(transition_probs[[i]]) <- list(from=state_names[[i]],to=state_names[[i]])
-      # Single channel but emission_probs is list of lists
-      if(is.list(emission_probs[[i]]) && length(emission_probs[[i]])==1)
-        emission_probs[[i]] <- emission_probs[[i]][[1]]
-    }
-
-
-
-
-    # Single channel but observations is a list
-    if (is.list(observations) && !inherits(observations, "stslist") && length(observations)==1) {
-      observations <- observations[[1]]
-    }
-
-    n_channels <- ifelse(is.list(emission_probs[[1]]), length(emission_probs[[1]]), 1)
-
-    for(i in 1:n_clusters){
-      if (n_channels == 1){
-        if (!is.matrix(emission_probs[[i]])) {
-          stop(paste("Object provided in emission_probs for cluster", i, "is not a matrix."))
+      
+      if (is.null(state_names)) {
+        state_names <- vector("list", n_clusters)
+        for(m in 1:n_clusters){
+          if (is.null(state_names[[m]] <- rownames(transition_probs[[m]]))) {
+            state_names[[m]] <-  paste("State", 1:n_states[m])
+          }
         }
       } else {
-        for (j in 1:n_channels){
-          if (!is.matrix(emission_probs[[i]][[j]])) {
-            stop(paste("Object provided in emission_probs for cluster", i, "and channel", j, "is not a matrix."))
+        for (m in 1:n_clusters) {
+          if (length(state_names[[m]]) != n_states[m]) {
+            stop(paste0("Length of state_names for cluster ", m, " is not equal to the number of hidden states."))
           }
         }
       }
-    }
-
-    if (n_channels>1 && any(sapply(emission_probs,length)!=n_channels)) {
-      stop("Number of channels defined by emission matrices differ from each other.")
-    }
-
-    if(n_channels>1){
-      if(length(observations)!=n_channels){
-        stop("Number of channels defined by emission_probs differs from one defined by observations.")
+      
+      for(i in 1:n_clusters){
+        if (!isTRUE(all.equal(rowSums(transition_probs[[i]]),
+                              rep(1, n_states[i]), check.attributes=FALSE))) {
+          stop(paste("Row sums of the transition probabilities in cluster", i, "do not sum to one."))
+        }
+        if (!isTRUE(all.equal(sum(initial_probs[[i]]), 1, check.attributes=FALSE))){
+          stop(paste("Initial state probabilities in cluster", i, "do not sum to one."))
+        }
       }
-
-      if (length(unique(sapply(observations, nrow))) > 1) {
-        stop("The number of subjects (rows) is not the same in all channels.")
+      
+      
+      
+      
+      for(i in 1:n_clusters){
+        dimnames(transition_probs[[i]]) <- list(from=state_names[[i]],to=state_names[[i]])
+        # Single channel but emission_probs is list of lists
+        if(is.list(emission_probs[[i]]) && length(emission_probs[[i]])==1)
+          emission_probs[[i]] <- emission_probs[[i]][[1]]
       }
-      if (length(unique(sapply(observations, ncol))) > 1) {
-        stop("The length of the sequences (number of columns) is not the same in all channels.")
+      
+      
+      
+      
+      # Single channel but observations is a list
+      if (is.list(observations) && !inherits(observations, "stslist") && length(observations)==1) {
+        observations <- observations[[1]]
       }
-
-      n_sequences <- nrow(observations[[1]])
-      length_of_sequences <- ncol(observations[[1]])
-
-
-      symbol_names <- lapply(observations, alphabet)
-      n_symbols <- lengths(symbol_names)
-      for (i in 1:n_clusters) {
-        if (length(initial_probs[[i]]) != n_states[i]) {
-          stop(paste("Length of initial_probs of cluster", i, "is not equal to the number of states."))
+      
+      n_channels <- ifelse(is.list(emission_probs[[1]]), length(emission_probs[[1]]), 1)
+      
+      for(i in 1:n_clusters){
+        if (n_channels == 1){
+          if (!is.matrix(emission_probs[[i]])) {
+            stop(paste("Object provided in emission_probs for cluster", i, "is not a matrix."))
+          }
+        } else {
+          for (j in 1:n_channels){
+            if (!is.matrix(emission_probs[[i]][[j]])) {
+              stop(paste("Object provided in emission_probs for cluster", i, "and channel", j, "is not a matrix."))
+            }
+          }
         }
-        if (any(lapply(emission_probs[[i]],nrow) != n_states[i])) {
-          stop(paste("Number of rows in emission_probs of cluster", i, "is not equal to the number of states."))
+      }
+      
+      if (n_channels>1 && any(sapply(emission_probs,length)!=n_channels)) {
+        stop("Number of channels defined by emission matrices differ from each other.")
+      }
+      
+      if(n_channels>1){
+        if(length(observations)!=n_channels){
+          stop("Number of channels defined by emission_probs differs from one defined by observations.")
         }
-
-        if (any(n_symbols != sapply(emission_probs[[i]],ncol))) {
-          stop(paste("Number of columns in emission_probs of cluster", i, "is not equal to the number of symbols."))
+        
+        if (length(unique(sapply(observations, nrow))) > 1) {
+          stop("The number of subjects (rows) is not the same in all channels.")
         }
-        if (!isTRUE(all.equal(c(sapply(emission_probs[[i]], rowSums)),
-                              rep(1, n_channels * n_states[i]), check.attributes = FALSE))) {
-          stop(paste("Emission probabilities in emission_probs of cluster", i, "do not sum to one."))
+        if (length(unique(sapply(observations, ncol))) > 1) {
+          stop("The length of the sequences (number of columns) is not the same in all channels.")
         }
+        
+        n_sequences <- nrow(observations[[1]])
+        length_of_sequences <- ncol(observations[[1]])
+        
+        
+        symbol_names <- lapply(observations, alphabet)
+        n_symbols <- lengths(symbol_names)
+        for (i in 1:n_clusters) {
+          if (length(initial_probs[[i]]) != n_states[i]) {
+            stop(paste("Length of initial_probs of cluster", i, "is not equal to the number of states."))
+          }
+          if (any(lapply(emission_probs[[i]],nrow) != n_states[i])) {
+            stop(paste("Number of rows in emission_probs of cluster", i, "is not equal to the number of states."))
+          }
+          
+          if (any(n_symbols != sapply(emission_probs[[i]],ncol))) {
+            stop(paste("Number of columns in emission_probs of cluster", i, "is not equal to the number of symbols."))
+          }
+          if (!isTRUE(all.equal(c(sapply(emission_probs[[i]], rowSums)),
+                                rep(1, n_channels * n_states[i]), check.attributes = FALSE))) {
+            stop(paste("Emission probabilities in emission_probs of cluster", i, "do not sum to one."))
+          }
+          if (is.null(channel_names)) {
+            if(is.null(channel_names <- names(observations))){
+              channel_names <- paste("Channel", 1:n_channels)
+            }
+          } else if (length(channel_names)!=n_channels) {
+            warning("The length of argument channel_names does not match the number of channels. Names were not used.")
+            channel_names<-paste("Channel", 1:n_channels)
+          }
+          for (j in 1:n_channels) {
+            dimnames(emission_probs[[i]][[j]])<-list(state_names=state_names[[i]],symbol_names=symbol_names[[j]])
+          }
+          names(emission_probs[[i]])<-channel_names
+          names(initial_probs[[i]]) <- state_names[[i]]
+        }
+      } else {
+        n_channels <- 1
         if (is.null(channel_names)) {
-          if(is.null(channel_names <- names(observations))){
+          channel_names <- "Observations"
+        }
+        n_sequences<-nrow(observations)
+        length_of_sequences<-ncol(observations)
+        symbol_names<-alphabet(observations)
+        n_symbols<-length(symbol_names)
+        
+        for(i in 1:n_clusters){
+          if(n_states[i]!=dim(emission_probs[[i]])[1])
+            stop("Number of rows in emission_probs is not equal to the number of states.")
+          if(n_symbols!=dim(emission_probs[[i]])[2])
+            stop("Number of columns in emission_probs is not equal to the number of symbols.")
+          if(!isTRUE(all.equal(rep(1,n_states[i]),rowSums(emission_probs[[i]]),check.attributes=FALSE)))
+            stop("Emission probabilities in emission_probs do not sum to one.")
+          dimnames(emission_probs[[i]])<-list(state_names=state_names[[i]],symbol_names=symbol_names)
+          names(initial_probs[[i]]) <- state_names[[i]]
+        }
+        
+      }
+      
+    # Simulate starting values
+    } else {
+      
+      n_clusters <- length(n_states)
+      
+      if (is.null(cluster_names)) {
+        cluster_names <- paste("Cluster", 1:n_clusters)
+      } else if (length(cluster_names) != n_clusters) {
+        warning("The length of argument cluster_names does not match the length of n_clusters. Names were not used.")
+        cluster_names <- paste("Cluster", 1:n_clusters)
+      }
+      
+      transition_probs <- simulate_transition_probs(n_states = n_states, n_clusters = n_clusters, ...)
+      
+      
+      if (is.null(state_names)) {
+        state_names <- vector("list", n_clusters)
+        for (m in 1:n_clusters) {
+          state_names[[m]] <-  paste("State", 1:n_states[m])
+        }
+      } else {
+        for (m in 1:n_clusters) {
+          if (length(state_names[[m]]) != n_states[m]) {
+            stop(paste0("Length of state_names for cluster ", m, " is not equal to the number of hidden states."))
+          }
+        }
+      }
+      for (k in 1:n_clusters) {
+        dimnames(transition_probs[[k]]) <- list(from = state_names[[k]], to = state_names[[k]])
+      }
+      
+      
+      initial_probs <- simulate_initial_probs(n_states = n_states, n_clusters = n_clusters)
+      
+      
+      if (inherits(observations, "stslist")) {
+        n_channels <- 1
+        
+        n_sequences <- nrow(observations)
+        length_of_sequences <- ncol(observations)
+        
+        symbol_names <- alphabet(observations)
+        n_symbols <- length(symbol_names)
+        
+        if (is.null(channel_names)) {
+          channel_names <- "Observations"
+        }
+        
+        emission_probs <- vector("list", n_clusters) 
+        for (k in 1:n_clusters){
+          emission_probs[[k]] <- simulate_emission_probs(n_states = n_states[k], n_symbols = n_symbols)
+          dimnames(emission_probs[[k]]) <- list(state_names = state_names[[k]], symbol_names = symbol_names)
+          names(initial_probs[[k]]) <- state_names[[k]]
+        }
+        
+      } else if (is.list(observations) && !inherits(observations, "stslist") && 
+                 length(observations) == 1) {
+          n_channels <- 1
+          
+          observations <- observations[[1]]
+          if (is.null(channel_names)) {
+            channel_names <- "Observations"
+          }
+          
+          n_sequences <- nrow(observations)
+          length_of_sequences <- ncol(observations)
+          
+          symbol_names <- alphabet(observations)
+          n_symbols <- length(symbol_names)
+          
+          
+          emission_probs <- vector("list", n_clusters) 
+          for (k in 1:n_clusters){
+            emission_probs[[k]] <- simulate_emission_probs(n_states = n_states[k], n_symbols = n_symbols)
+            dimnames(emission_probs[[k]]) <- list(state_names = state_names[[k]], symbol_names = symbol_names)
+            names(initial_probs[[k]]) <- state_names[[k]]
+          }
+        } else {
+        n_channels <- length(observations)
+        
+        n_sequences <- nrow(observations[[1]])
+        length_of_sequences <- ncol(observations[[1]])
+        
+        symbol_names <- lapply(observations, alphabet)
+        n_symbols <- lengths(symbol_names)
+        
+        if (is.null(channel_names)) {
+          if (is.null(channel_names <- names(observations))){
             channel_names <- paste("Channel", 1:n_channels)
           }
-        } else if (length(channel_names)!=n_channels) {
+        } else if (length(channel_names) != n_channels) {
           warning("The length of argument channel_names does not match the number of channels. Names were not used.")
-          channel_names<-paste("Channel", 1:n_channels)
+          channel_names <- paste("Channel", 1:n_channels)
         }
-        for (j in 1:n_channels) {
-          dimnames(emission_probs[[i]][[j]])<-list(state_names=state_names[[i]],symbol_names=symbol_names[[j]])
+        
+        emission_probs <- vector("list", n_clusters) 
+        for (k in 1:n_clusters) {
+          emission_probs[[k]] <- vector("list", n_channels) 
+          for (c in 1:n_channels) {
+            emission_probs[[k]][[c]] <- simulate_emission_probs(n_states = n_states[k], n_symbols = n_symbols[c])
+            dimnames(emission_probs[[k]][[c]]) <- list(state_names = state_names[[k]], symbol_names = symbol_names[[c]])
+          }
+          names(emission_probs[[k]]) <- channel_names
+          names(initial_probs[[k]]) <- state_names[[k]]
         }
-        names(emission_probs[[i]])<-channel_names
-        names(initial_probs[[i]]) <- state_names[[i]]
+        
+        
       }
-    } else {
-      n_channels <- 1
-      if (is.null(channel_names)) {
-        channel_names <- "Observations"
-      }
-      n_sequences<-nrow(observations)
-      length_of_sequences<-ncol(observations)
-      symbol_names<-alphabet(observations)
-      n_symbols<-length(symbol_names)
-
-      for(i in 1:n_clusters){
-        if(n_states[i]!=dim(emission_probs[[i]])[1])
-          stop("Number of rows in emission_probs is not equal to the number of states.")
-        if(n_symbols!=dim(emission_probs[[i]])[2])
-          stop("Number of columns in emission_probs is not equal to the number of symbols.")
-        if(!isTRUE(all.equal(rep(1,n_states[i]),rowSums(emission_probs[[i]]),check.attributes=FALSE)))
-          stop("Emission probabilities in emission_probs do not sum to one.")
-        dimnames(emission_probs[[i]])<-list(state_names=state_names[[i]],symbol_names=symbol_names)
-        names(initial_probs[[i]]) <- state_names[[i]]
-      }
-
     }
+    
+    
+    
     if(missing(formula)){
       formula <- stats::formula(rep(1, n_sequences) ~ 1)
     }
@@ -389,11 +523,11 @@ build_mhmm <-
         stop("Wrong dimensions of coefficients.")
       coefficients[,1]<-0
     }
-
-
+    
+    
     rownames(coefficients) <- colnames(X)
     colnames(coefficients) <- cluster_names
-
+    
     names(transition_probs) <- names(emission_probs) <- names(initial_probs) <- cluster_names
     if(n_channels > 1){
       nobs <- sum(sapply(observations, function(x) sum(!(x == attr(observations[[1]], "nr") |
@@ -416,6 +550,6 @@ build_mhmm <-
                        nobs = nobs,
                        df = sum(unlist(initial_probs) > 0) - n_clusters + sum(unlist(transition_probs) > 0) - sum(n_states) +
                          sum(unlist(emission_probs) > 0) - sum(n_states) * n_channels + n_covariates * (n_clusters - 1),
-      type = "mhmm")
+                       type = "mhmm")
     model
   }
