@@ -38,8 +38,10 @@
 #'   for the original EM phase, the final model is re-estimated with the original \code{reltol}
 #'   and \code{maxeval} at the end of the EM step.}
 #'   \item{n_optimum}{Save the log-likelihood values of the \code{n_optimum} best
-#'   models (from all estimated models including the original).
+#'   models (from all estimated models including the the first EM run.).
 #'   The default is \code{min(times + 1, 25)}.}
+#'   \item{use_original}{If \code{TRUE}. Use the initial values of the input model as starting 
+#'   points for the permutations. Otherwise permute the results of the first EM run.}
 #'   }
 #'   }
 #'   }
@@ -53,7 +55,7 @@
 #'}
 #' @param lb,ub Lower and upper bounds for parameters in Softmax parameterization.
 #' The default interval is \eqn{[pmin(-25, 2*initialvalues), pmax(25, 2*initialvalues)]},
-#' except for beta coefficients,
+#' except for gamma coefficients,
 #' where the scale of covariates is taken into account.
 #' Note that it might still be a good idea to scale covariates around unit scale.
 #' Bounds are used only in the global optimization step.
@@ -172,13 +174,10 @@
 #' data("hmm_mvad")
 #'
 #' # Markov model
+#' # Note: build_mm estimates model parameters from observations,
+#' # no need for estimating with fit_model
 #'
-#' set.seed(123)
-#' init_mm_mvad <- build_mm(observations = mvad_seq,
-#'   transition_probs = simulate_transition_probs(6),
-#'   initial_probs = rep(1/6, 6))
-#'
-#' mm_mvad <- fit_model(init_mm_mvad)$model
+#' mm_mvad <- build_mm(observations = mvad_seq)
 #'
 #' # Comparing likelihoods, MM fits better
 #' logLik(hmm_mvad)
@@ -432,33 +431,33 @@
 fit_model <- function(model, em_step = TRUE, global_step = FALSE, local_step = FALSE,
   control_em = list(), control_global = list(), control_local = list(), lb, ub, threads = 1,
   log_space = FALSE, ...){
-
-
+  
+  
   if (!inherits(model, c("hmm", "mhmm"))){
     stop("Argument model must be an object of class 'hmm' or 'mhmm.")
   }
-
+  
   if (!em_step && !global_step && !local_step) {
     stop("No method chosen for estimation. Choose at least one from em_step, global_step, and local_step.")
   }
   if (threads < 1) {
     stop("Argument threads must be a positive integer.")
   }
-
+  
   mhmm <- inherits(model, c("mhmm"))
-
+  
   if (mhmm) {
     df <- attr(model, "df")
     nobs <- attr(model, "nobs")
     original_model <- model
     model <- combine_models(model)
   }
-
+  
   if (model$n_channels == 1) {
     model$observations <- list(model$observations)
     model$emission_probs <- list(model$emission_probs)
   }
-
+  
   obsArray <- array(0, c(model$n_sequences, model$length_of_sequences,
     model$n_channels))
   for(i in 1:model$n_channels) {
@@ -466,19 +465,19 @@ fit_model <- function(model, em_step = TRUE, global_step = FALSE, local_step = F
     obsArray[,,i][obsArray[,,i] > model$n_symbols[i]] <- model$n_symbols[i]
   }
   obsArray <- aperm(obsArray)
-
+  
   emissionArray<-array(1,c(model$n_states,max(model$n_symbols)+1,model$n_channels))
   for(i in 1:model$n_channels)
     emissionArray[,1:model$n_symbols[i],i]<-model$emission_probs[[i]]
-
+  
   if (em_step) {
     em.con <- list(print_level = 0, maxeval = 1000, reltol = 1e-10, restart = NULL)
     nmsC <- names(em.con)
     em.con[(namc <- names(control_em))] <- control_em
     if (length(noNms <- namc[!namc %in% nmsC]))
       warning("Unknown names in control_em: ", paste(noNms, collapse = ", "))
-
-
+    
+    
     if (!log_space) {
       if (mhmm) {
         resEM <- EMx(model$transition_probs, emissionArray, model$initial_probs, obsArray,
@@ -498,15 +497,15 @@ fit_model <- function(model, em_step = TRUE, global_step = FALSE, local_step = F
           model$n_symbols, em.con$maxeval, em.con$reltol,em.con$print_level, threads)
       }
     }
-
-
+    
+    
     if (resEM$error != 0) {
       err_msg <- switch(resEM$error,
         "Scaling factors contain non-finite values.",
         "Backward probabilities contain non-finite values.",
         "Initial values of coefficients of covariates gives non-finite cluster probabilities.",
-        "Estimation of coefficients of covariates failed due to singular Hessian.",
-        "Estimation of coefficients of covariates failed due to non-finite cluster probabilities.",
+        "Estimation of gamma coefficients failed due to singular Hessian.",
+        "Estimation of gamma coefficients failed due to non-finite cluster probabilities.",
         "Non-finite log-likelihood")
       if (!global_step && !local_step &&
           (is.null(control_em$restart$times) || control_em$restart$times == 0)) {
@@ -514,61 +513,53 @@ fit_model <- function(model, em_step = TRUE, global_step = FALSE, local_step = F
       } else warning(paste("EM algorithm failed:", err_msg))
       resEM$logLik <- -Inf
     }
-
-
+    
+    
     if (!is.null(em.con$restart)) {
       restart.con <- list(times = 0, print_level = em.con$print_level,
         maxeval = em.con$maxeval, reltol = em.con$reltol,
         transition = TRUE, emission = TRUE, coef = FALSE, sd = 0.25,
-        n_optimum = min(control_em$restart$times + 1, 25))
+        n_optimum = min(control_em$restart$times + 1, 25), use_original = TRUE)
       nmsC <- names(restart.con)
       restart.con[(namc <- names(control_em$restart))] <- control_em$restart
       if (length(noNms <- namc[!namc %in% nmsC]))
         warning("Unknown names in control_em$restart: ", paste(noNms, collapse = ", "))
       restart.con$n_optimum <- min(restart.con$n_optimum, restart.con$times + 1)
     }
-
-
+    
+    
     if (!is.null(em.con$restart) && restart.con$times > 0 &&
         (restart.con$transition | restart.con$emission)) {
-
+      
       opt_restart <- numeric(restart.con$times + 1)
       opt_restart[1] <- resEM$logLik
-
-      if (resEM$error == 0) {
-        random_emiss <- resEM$emissionArray
-        random_emiss[(random_emiss < 1e-4) & (emissionArray > 0)] <- 1e-4
-        for (j in 1:model$n_channels) {
-          random_emiss[,1:model$n_symbols[j],j] <-
-            random_emiss[,1:model$n_symbols[j],j] / rowSums(random_emiss[,1:model$n_symbols[j],j])
-        }
-        random_trans <- resEM$transitionMatrix
-        random_trans[(random_trans < 1e-4) & (model$transition_probs > 0)] <- 1e-4
-        random_trans <- random_trans / rowSums(random_trans)
-        random_coef <- resEM$coefficients
-      } else {
+      
+      if (restart.con$use_original) {
         random_emiss <- emissionArray
         random_trans <- model$transition_probs
         random_coef <- model$coefficients
+      } else {
+        random_emiss <- resEM$emission
+        random_trans <- resEM$transition
+        random_coef <- resEM$coef
       }
-
       if (restart.con$transition) {
         nz_trans <- (random_trans > 0 & random_trans < 1)
         np_trans <- sum(nz_trans)
         base_trans <- random_trans[nz_trans]
-      }
+      } 
       if (restart.con$emission) {
         nz_emiss <- (random_emiss > 0 & random_emiss < 1)
         np_emiss <- sum(nz_emiss)
         base_emiss <- random_emiss[nz_emiss]
       }
-
+      
       if (restart.con$coef) {
         base_coef <- random_coef
         coef_scale <- max(abs(base_coef))
         n_coef <- length(base_coef)
       }
-
+      
       for (i in 1:restart.con$times) {
         if (restart.con$transition) {
           random_trans[nz_trans] <- abs(base_trans + rnorm(np_trans, sd = restart.con$sd))
@@ -586,6 +577,7 @@ fit_model <- function(model, em_step = TRUE, global_step = FALSE, local_step = F
         if (restart.con$coef) {
           random_coef[] <- rnorm(n_coef, base_coef, sd = coef_scale)
         }
+        
         if (!log_space) {
           if (mhmm) {
             resEMi <- EMx(random_trans, random_emiss, model$initial_probs, obsArray,
@@ -605,17 +597,16 @@ fit_model <- function(model, em_step = TRUE, global_step = FALSE, local_step = F
               model$n_symbols, restart.con$maxeval, restart.con$reltol,restart.con$print_level, threads)
           }
         }
-
-
+        
+        
         if (resEMi$error != 0) {
           opt_restart[i + 1] <- -Inf
           err_msg <- switch(resEMi$error,
             "Scaling factors contain non-finite values.",
             "Backward probabilities contain non-finite values.",
             "Initial values of coefficients of covariates gives non-finite cluster probabilities.",
-            "Estimation of coefficients of covariates failed due to singular Hessian.",
-            "Estimation of coefficients of covariates failed due to non-finite cluster probabilities.",
-            "Non-finite log-likelihood")
+            "Estimation of gamma coefficients failed due to singular Hessian.",
+            "Estimation of gamma coefficients failed due to non-finite cluster probabilities.", "Non-finite log-likelihood")
           warning(paste("EM algorithm failed:", err_msg))
         } else {
           opt_restart[i + 1] <- resEMi$logLik
@@ -624,9 +615,9 @@ fit_model <- function(model, em_step = TRUE, global_step = FALSE, local_step = F
           }
         }
       }
-
+      
       opts <- sort(opt_restart, decreasing = TRUE)[1:restart.con$n_optimum]
-
+      
       if (em.con$reltol < resEM$change) {
         if (!log_space) {
           if (mhmm) {
@@ -637,7 +628,7 @@ fit_model <- function(model, em_step = TRUE, global_step = FALSE, local_step = F
             resEM <- EM(resEM$transitionMatrix, resEM$emissionArray, resEM$initialProbs, obsArray,
               model$n_symbols, em.con$maxeval, em.con$reltol,em.con$print_level, threads)
           }
-
+          
         } else {
           if (mhmm) {
             resEM <- log_EMx(resEM$transitionMatrix, resEM$emissionArray, resEM$initialProbs, obsArray,
@@ -649,19 +640,19 @@ fit_model <- function(model, em_step = TRUE, global_step = FALSE, local_step = F
           }
         }
       }
-
+      
     } else {
       opts <- resEM$logLik
     }
-
+    
     if (resEM$error == 0) {
       if (resEM$change < -em.con$reltol)
         warning("EM algorithm stopped due to decreasing log-likelihood. ")
-
+      
       emissionArray <- resEM$emissionArray
       for (i in 1:model$n_channels)
         model$emission_probs[[i]][] <- emissionArray[ , 1:model$n_symbols[i], i]
-
+      
       if (mhmm && (global_step || local_step)) {
         k <- 0
         for (m in 1:model$n_clusters) {
@@ -672,7 +663,7 @@ fit_model <- function(model, em_step = TRUE, global_step = FALSE, local_step = F
       } else {
         model$initial_probs[] <- resEM$initialProbs
       }
-
+      
       resEM$best_opt_restart <- opts
       model$transition_probs[] <- resEM$transitionMatrix
       model$coefficients[] <- resEM$coefficients
@@ -682,9 +673,9 @@ fit_model <- function(model, em_step = TRUE, global_step = FALSE, local_step = F
       ll <- -Inf
     }
   } else resEM <- NULL
-
+  
   if (global_step || local_step){
-
+    
     # Largest transition probabilities (for each row)
     x <- which(model$transition_probs > 0, arr.ind = TRUE)
     transNZ <- x[order(x[,1]),]
@@ -695,13 +686,13 @@ fit_model <- function(model, em_step = TRUE, global_step = FALSE, local_step = F
     npTM <- nrow(paramTM)
     transNZ <- model$transition_probs > 0
     transNZ[maxTM] <- 0
-
-
+    
+    
     emissNZ <- lapply(model$emission_probs, function(i) {
       x <- which(i > 0, arr.ind = TRUE)
       x[order(x[, 1]), ]
     })
-
+    
     if (model$n_states > 1) {
       maxEM <- lapply(model$emission_probs, function(i){
         cbind(1:model$n_states,max.col(i, ties.method = "first"))
@@ -721,13 +712,13 @@ fit_model <- function(model, em_step = TRUE, global_step = FALSE, local_step = F
     }
     maxEMvalue <- lapply(1:model$n_channels, function(i)
       apply(model$emission_probs[[i]],1,max))
-
+    
     emissNZ <- array(0, c(model$n_states,max(model$n_symbols), model$n_channels))
     for (i in 1:model$n_channels) {
       emissNZ[,1:model$n_symbols[i],i]<-model$emission_probs[[i]] > 0
       emissNZ[,1:model$n_symbols[i],i][maxEM[[i]]]<-0
     }
-
+    
     if (mhmm) {
       maxIP <- maxIPvalue <- npIP <- numeric(original_model$n_clusters)
       paramIP <-  initNZ <- vector("list",original_model$n_clusters)
@@ -744,10 +735,10 @@ fit_model <- function(model, em_step = TRUE, global_step = FALSE, local_step = F
       }
       initNZ<-unlist(initNZ)
       npIPAll <- sum(unlist(npIP))
-
+      
       npCoef<-length(model$coefficients[,-1])
       model$coefficients[,1] <- 0
-
+      
       initialvalues<-c(if((npTM+sum(npEM)+npIPAll)>0) log(c(
         if(npTM>0) model$transition_probs[paramTM],
         if(sum(npEM)>0) unlist(sapply(1:model$n_channels,
@@ -757,16 +748,16 @@ fit_model <- function(model, em_step = TRUE, global_step = FALSE, local_step = F
       )),
         model$coefficients[,-1]
       )
-
+      
     } else {
       maxIP <- which.max(model$initial_probs)
       maxIPvalue <- model$initial_probs[maxIP]
       paramIP <- setdiff(which(model$initial_probs > 0), maxIP)
-
+      
       npIP <- length(paramIP)
       initNZ <- model$initial_probs > 0
       initNZ[maxIP] <- 0
-
+      
       initialvalues<-c(if((npTM+sum(npEM)+npIP)>0) log(c(
         if(npTM>0) model$transition_probs[paramTM],
         if(sum(npEM)>0) unlist(sapply(1:model$n_channels,
@@ -774,10 +765,10 @@ fit_model <- function(model, em_step = TRUE, global_step = FALSE, local_step = F
         if(npIP>0) model$initial_probs[paramIP]))
       )
     }
-
-
+    
+    
     objectivef_mhmm<-function(pars, model, estimate = TRUE){
-
+      
       if(any(!is.finite(exp(pars))) && estimate)
         return(list(objective = Inf, gradient = rep(-Inf, length(pars))))
       if(npTM>0){
@@ -805,7 +796,7 @@ fit_model <- function(model, em_step = TRUE, global_step = FALSE, local_step = F
       }
       model$initial_probs <- unlist(original_model$initial_probs)
       model$coefficients[,-1] <- pars[npTM+sum(npEM)+npIPAll+1:npCoef]
-
+      
       if (estimate) {
         if (!log_space) {
           if (need_grad) {
@@ -821,13 +812,13 @@ fit_model <- function(model, em_step = TRUE, global_step = FALSE, local_step = F
             log_objectivex(model$transition_probs, emissionArray, model$initial_probs, obsArray,
               transNZ, emissNZ, initNZ, model$n_symbols,
               model$coefficients, model$X, model$n_states_in_clusters, threads)
-
+            
           } else {
             -sum(log_logLikMixHMM(model$transition_probs, emissionArray, model$initial_probs, obsArray,
               model$coefficients, model$X, model$n_states_in_clusters, threads))
           }
         }
-
+        
       } else {
         if (sum(npEM) > 0) {
           for (i in 1:model$n_channels) {
@@ -837,9 +828,9 @@ fit_model <- function(model, em_step = TRUE, global_step = FALSE, local_step = F
         model
       }
     }
-
+    
     objectivef_hmm <- function(pars, model, estimate = TRUE){
-
+      
       if(any(!is.finite(exp(pars))) && estimate)
         return(list(objective = Inf, gradient = rep(-Inf, length(pars))))
       if(npTM>0){
@@ -856,14 +847,14 @@ fit_model <- function(model, em_step = TRUE, global_step = FALSE, local_step = F
             emissionArray[,1:model$n_symbols[i],i]/rowSums(emissionArray[,1:model$n_symbols[i],i, drop = FALSE])
         }
       }
-
+      
       if(npIP>0){
         model$initial_probs[maxIP]<-maxIPvalue
         model$initial_probs[paramIP]<-exp(pars[(npTM+sum(npEM)+1):(npTM+sum(npEM)+npIP)])
         model$initial_probs[]<-model$initial_probs/sum(model$initial_probs)
       }
-
-
+      
+      
       if (estimate) {
         if (!log_space) {
           if (need_grad) {
@@ -882,7 +873,7 @@ fit_model <- function(model, em_step = TRUE, global_step = FALSE, local_step = F
               model$initial_probs, obsArray, threads))
           }
         }
-
+        
       } else {
         if (sum(npEM) > 0) {
           for (i in 1:model$n_channels) {
@@ -892,15 +883,15 @@ fit_model <- function(model, em_step = TRUE, global_step = FALSE, local_step = F
         model
       }
     }
-
+    
     if (global_step) {
-
+      
       if (missing(lb)) {
         if (mhmm) {
           lb <- c(rep(-25, length(initialvalues) - npCoef),
             rep(-150 / apply(abs(model$X), 2, max), model$n_clusters - 1))
         } else lb <- -25
-
+        
       }
       lb <- pmin(lb, 2*initialvalues)
       if (missing(ub)) {
@@ -910,7 +901,7 @@ fit_model <- function(model, em_step = TRUE, global_step = FALSE, local_step = F
         } else ub <- 25
       }
       ub <- pmin(pmax(ub, 2*initialvalues),500)
-
+      
       if (is.null(control_global$maxeval)) {
         control_global$maxeval <- 10000
       }
@@ -925,7 +916,7 @@ fit_model <- function(model, em_step = TRUE, global_step = FALSE, local_step = F
         }
       }
       need_grad <- grepl("NLOPT_GD_",control_global$algorithm)
-
+      
       if (mhmm) {
         globalres <- nloptr(x0 = initialvalues, eval_f = objectivef_mhmm, lb = lb, ub = ub,
           opts = control_global, model = model, estimate = TRUE, ...)
@@ -935,15 +926,15 @@ fit_model <- function(model, em_step = TRUE, global_step = FALSE, local_step = F
           opts = control_global, model = model, estimate = TRUE,  ...)
         model <- objectivef_hmm(globalres$solution, model, FALSE)
       }
-
+      
       if (globalres$status < 0) {
         warning(paste("Global optimization terminated:", globalres$message))
       }
       initialvalues <- globalres$solution
       ll <- -globalres$objective
-
+      
     } else globalres <- NULL
-
+    
     if (local_step) {
       if ( is.null(control_local$maxeval)) {
         control_local$maxeval <- 10000
@@ -957,13 +948,13 @@ fit_model <- function(model, em_step = TRUE, global_step = FALSE, local_step = F
       if (is.null(control_local$ftol_rel)) {
         control_local$ftol_rel <- 1e-10
       }
-
+      
       need_grad <- grepl("NLOPT_LD_",control_local$algorithm)
       if (mhmm) {
         localres <- nloptr(x0 = initialvalues, eval_f = objectivef_mhmm,
           opts = control_local, model = model, estimate = TRUE, ...)
         model <- objectivef_mhmm(localres$solution, model, FALSE)
-
+        
       } else {
         localres <- nloptr(x0 = initialvalues, eval_f = objectivef_hmm,
           opts = control_local, model = model, estimate = TRUE, ...)
@@ -974,24 +965,24 @@ fit_model <- function(model, em_step = TRUE, global_step = FALSE, local_step = F
       }
       ll <- -localres$objective
     } else localres <- NULL
-
+    
     if (mhmm) {
       rownames(model$coefficients) <- colnames(model$X)
       colnames(model$coefficients) <- model$cluster_names
     }
   } else globalres <- localres <- NULL
-
+  
   if (model$n_channels == 1) {
     model$observations <- model$observations[[1]]
     model$emission_probs <- model$emission_probs[[1]]
   }
-
+  
   if (mhmm) {
     model <- spread_models(model)
     attr(model, "df") <- df
     attr(model, "nobs") <- nobs
     attr(model, "type") <- attr(original_model, "type")
-
+    
     for (i in 1:model$n_clusters) {
       dimnames(model$transition_probs[[i]]) <- dimnames(original_model$transition_probs[[i]])
       for (j in 1:model$n_channels) {
@@ -999,10 +990,10 @@ fit_model <- function(model, em_step = TRUE, global_step = FALSE, local_step = F
       }
     }
   }
-
+  
   suppressWarnings(try(model <- trim_model(model, verbose = FALSE), silent = TRUE))
   list(model = model,
     logLik = ll, em_results = resEM[c("logLik", "iterations", "change", "best_opt_restart")],
     global_results = globalres, local_results = localres, call = match.call())
-
+  
 }
