@@ -69,7 +69,7 @@
 #'   \code{"combine.slices"} is greater than 0). The default color is white.
 #' @param combined.slice.label The label for combined states (when argument
 #'   \code{"combine.slices"} is greater than 0) to appear in the legend.
-#' @param withlegend Defines if and where the legend of state colors is
+#' @param with.legend Defines if and where the legend of state colors is
 #'   plotted. Possible values include \code{"bottom"} (the default),
 #'   \code{"top"}, \code{"left"}, and \code{"right"}. \code{FALSE} omits the
 #'   legend.
@@ -89,6 +89,7 @@
 #'   specified for all (combinations of) observed states even if they are not
 #'   plotted (if the probability is less than \code{combine.slices}).
 #' @param main Main title for the plot. Omitted by default.
+#' @param withlegend Deprecated. Use \code{with.legend} instead.
 #' @param ... Other parameters passed on to \code{\link{plot.igraph}} such as
 #'   \code{vertex.color}, \code{vertex.label.cex}, or \code{edge.lty}.
 #'
@@ -133,7 +134,7 @@
 #'   # varying positions for vertex labels (initial probabilities)
 #'   vertex.label.pos = c(pi, pi/2, -pi/2, 0, pi/2),
 #'   # different legend properties
-#'   withlegend = "top", legend.prop = 0.3, cex.legend = 1.1,
+#'   with.legend = "top", legend.prop = 0.3, cex.legend = 1.1,
 #'   # Fix axes to the right scale
 #'   xlim = c(0.5, 5.5), ylim = c(-1.5, 1.5), rescale = FALSE,
 #'   # all states (not combining states with small probabilities)
@@ -206,11 +207,13 @@ plot.hmm  <- function(x, layout = "horizontal", pie = TRUE,
                       trim = 1e-15,
                       combine.slices = 0.05, combined.slice.color = "white",
                       combined.slice.label = "others",
-                      withlegend = "bottom", ltext = NULL, legend.prop = 0.5,
+                      with.legend = "bottom", ltext = NULL, legend.prop = 0.5,
                       cex.legend = 1, ncol.legend = "auto", cpal = "auto",
-                      main = NULL, ...){
-
+                      main = NULL, withlegend, ...){
 
+  
+  check_deprecated_args(match.call())
+  
   # Saving and changing marginals
   oldPar  <- par(no.readonly = TRUE)
 
@@ -251,13 +254,13 @@ plot.hmm  <- function(x, layout = "horizontal", pie = TRUE,
   }
 
   choices <- c(TRUE, FALSE, "bottom", "top", "left", "right")
-  ind <- pmatch(withlegend, choices)
+  ind <- pmatch(with.legend, choices)
   if (is.na(ind)) {
-    stop("Argument withlegend must be one of TRUE, FALSE, \"bottom\", \"right\", \"top\", or \"left\".")
+    stop("Argument with.legend must be one of TRUE, FALSE, \"bottom\", \"right\", \"top\", or \"left\".")
   }
-  withlegend <- choices[ind]
-  if (withlegend %in% c(TRUE, "auto")){
-    withlegend <- "bottom"
+  with.legend <- choices[ind]
+  if (with.legend %in% c(TRUE, "auto")){
+    with.legend <- "bottom"
   }
 
 
@@ -267,8 +270,8 @@ plot.hmm  <- function(x, layout = "horizontal", pie = TRUE,
   }
 
   # No slices -> no legends needed
-  if (pie == FALSE && withlegend != FALSE) {
-    withlegend  <- FALSE
+  if (pie == FALSE && with.legend != FALSE) {
+    with.legend  <- FALSE
   }
 
 
@@ -307,7 +310,7 @@ plot.hmm  <- function(x, layout = "horizontal", pie = TRUE,
     if (is.na(ind)) {
       stop("Argument vertex.label.dist only accepts the value \"auto\" or a numerical vector.")
     }
-    vertex.label.dist  <- vertex.size * 0.4 / 10
+    vertex.label.dist  <- vertex.size * 0.4 / 3.5
   } else if (length(vertex.label.dist) > 1 && length(vertex.label.dist) != x$n_states){
     warning("The length of the vector provided for the argument \"vertex.label.dist\" does not match the number of edges.")
     vertex.label.dist  <- rep(vertex.label.dist, length.out = length(x$n_states))
@@ -384,12 +387,12 @@ plot.hmm  <- function(x, layout = "horizontal", pie = TRUE,
   } else {
     pie.colors  <- cpal
   }
-  if (withlegend != FALSE) {
+  if (with.legend != FALSE) {
     pie.colors.l  <- pie.colors
   }
 
   # Legend position and number of columns
-  if (withlegend != FALSE && pie == TRUE) {
+  if (with.legend != FALSE && pie == TRUE) {
     if (!is.null(ltext)) {
       if (length(ltext) != x$n_symbols) {
         stop("The length of the argument ltext does not match the number of (combined) observed states.")
@@ -398,11 +401,11 @@ plot.hmm  <- function(x, layout = "horizontal", pie = TRUE,
     } else {
       ltext  <- x$symbol_names
     }
-    if (withlegend == "bottom") {
+    if (with.legend == "bottom") {
       graphics::layout(matrix(1:2, nrow = 2), heights = c(1 - legend.prop, legend.prop))
-    } else if (withlegend == "right") {
+    } else if (with.legend == "right") {
       graphics::layout(matrix(1:2, nrow = 1), widths = c(1 - legend.prop, legend.prop))
-    } else if (withlegend == "left") {
+    } else if (with.legend == "left") {
       graphics::layout(matrix(2:1, nrow = 1), widths = c(legend.prop, 1 - legend.prop))
     } else {
       graphics::layout(matrix(2:1, nrow = 2), widths = c(legend.prop, 1 - legend.prop))
@@ -476,7 +479,7 @@ plot.hmm  <- function(x, layout = "horizontal", pie = TRUE,
     # If slices are combined
     if (combine.slices > 0 &&
         !all(unlist(pie.values)[unlist(pie.values) > 0] > combine.slices)) {
-      if (withlegend != FALSE) {
+      if (with.legend != FALSE) {
         pie.colors.l  <- NULL
         lt  <- NULL
       }
@@ -488,17 +491,17 @@ plot.hmm  <- function(x, layout = "horizontal", pie = TRUE,
         # Colors and labels for large slices
         pie.values[[i]]  <- c(pie.values[[i]], cs.prob)
         # Texts and colors for legends
-        if (withlegend != FALSE) {
+        if (with.legend != FALSE) {
           pie.colors.l  <- c(pie.colors.l, pie.colors[pie.values[[i]][1:(length(pie.values[[i]]) - 1)] >= combine.slices])
           lt  <- c(lt, ltext[pie.values[[i]][1:(length(pie.values[[i]]) - 1)] >= combine.slices])
         }
       }
-      if (withlegend != FALSE) {
+      if (with.legend != FALSE) {
         ltext  <- c(unique(lt), combined.slice.label)
         pie.colors.l  <- c(unique(pie.colors.l), combined.slice.color)
       }
       if (ncol.legend == "auto") {
-        if (withlegend == "bottom" || withlegend == "top") {
+        if (with.legend == "bottom" || with.legend == "top") {
           ncol.legend  <- ceiling(length(pie.colors) / 4)
         } else {
           ncol.legend  <- 1
@@ -508,7 +511,7 @@ plot.hmm  <- function(x, layout = "horizontal", pie = TRUE,
       # Slices not combined
     } else {
       if (ncol.legend == "auto") {
-        if (withlegend == "bottom" || withlegend == "top") {
+        if (with.legend == "bottom" || with.legend == "top") {
           ncol.legend  <- ceiling(ncol(x$emission_probs) / 4)
         } else {
           ncol.legend  <- 1
@@ -518,7 +521,7 @@ plot.hmm  <- function(x, layout = "horizontal", pie = TRUE,
 
     if (!is.matrix(layout) && !is.function(layout) &&
         (layout == "horizontal" || layout == "vertical")) {
-      do.call(plot.igraph2, c(list(g1, layout = glayout,
+      do.call(plot.igraph, c(list(g1, layout = glayout,
                                    vertex.shape = "pie", vertex.pie = pie.values,
                                    vertex.pie.color = list(pie.colors),
                                    vertex.size = vertex.size,
@@ -532,7 +535,7 @@ plot.hmm  <- function(x, layout = "horizontal", pie = TRUE,
                                    xlim = xlim, ylim = ylim, rescale = rescale,
                                    main = main), dots))
     } else {
-      do.call(plot.igraph2, c(list(g1, layout = glayout,
+      do.call(plot.igraph, c(list(g1, layout = glayout,
                                    vertex.shape = "pie", vertex.pie = pie.values,
                                    vertex.pie.color = list(pie.colors),
                                    vertex.size = vertex.size,
@@ -548,7 +551,7 @@ plot.hmm  <- function(x, layout = "horizontal", pie = TRUE,
   # pie = FALSE
   } else {
     if(!is.matrix(layout) && !is.function(layout) && (layout == "horizontal" || layout == "vertical")){
-      do.call(plot.igraph2, c(list(g1, layout = glayout,
+      do.call(plot.igraph, c(list(g1, layout = glayout,
                                    vertex.size = vertex.size,
                                    vertex.label = vertex.label, vertex.label.dist = vertex.label.dist,
                                    vertex.label.degree = vertex.label.pos,
@@ -559,7 +562,7 @@ plot.hmm  <- function(x, layout = "horizontal", pie = TRUE,
                                    xlim = xlim, ylim = ylim, rescale = rescale,
                                    main = main), dots))
     } else {
-      do.call(plot.igraph2, c(list(g1, layout = glayout,
+      do.call(plot.igraph, c(list(g1, layout = glayout,
                                    vertex.size = vertex.size,
                                    vertex.label = vertex.label, vertex.label.dist = vertex.label.dist,
                                    vertex.label.degree = vertex.label.pos,
@@ -573,9 +576,9 @@ plot.hmm  <- function(x, layout = "horizontal", pie = TRUE,
 
 
   # Plotting legend
-  if (withlegend != FALSE && pie == TRUE) {
+  if (with.legend != FALSE && pie == TRUE) {
     seqlegend(x$observations, cpal = pie.colors.l, ltext = ltext,
-              position = "center", fontsize = cex.legend, ncol = ncol.legend,
+              position = "center", cex = cex.legend, ncol = ncol.legend,
               with.missing = FALSE)
   }
 
